# -*- coding: utf-8 -*-
import json
import logging

from .client import RequestAPIClient
from .conf import APP_CODE, SECRET_KEY
from .exceptions import APIException

logger = logging.getLogger(__name__)

__all__ = [
    "get_client_by_request",
    "get_client_by_user",
]


HEADER_BK_AUTHORIZATION = "X-Bkapi-Authorization"


def get_client_by_request(request, stage="prod", common_args=None, headers={}):
    """根据当前请求返回一个client
    :param request: 一个django request实例
    :param stage: 请求环境，默认为prod
    :param common_args: 公共请求参数
    :param headers: 头部信息
    :returns: 一个初始化好的APIClint对象
    """
    is_authenticated = request.user.is_authenticated
    if callable(is_authenticated):
        is_authenticated = is_authenticated()
    if is_authenticated:
        try:
            from bkoauth import get_access_token

            access_token = get_access_token(request)
            headers.update(
                {
                    HEADER_BK_AUTHORIZATION: json.dumps({"access_token": access_token.access_token}),
                }
            )
        except Exception:
            pass
    else:
        raise APIException(u"用户未通过验证")

    return RequestAPIClient(
        app_code=APP_CODE, app_secret=SECRET_KEY, headers=headers, common_args=common_args, stage=stage
    )


def get_client_by_user(user, stage="prod", common_args=None, headers={}):
    """根据user实例返回一个client
    :param user: 用户
    :param stage: 请求环境，默认为prod
    :param common_args: 公共请求参数
    :param headers: 头部信息
    :returns: 一个初始化好的APIClint对象
    """
    if hasattr(user, "username"):
        user = user.username
    try:
        from bkoauth import get_access_token_by_user

        access_token = get_access_token_by_user(user)
        headers.update(
            {
                HEADER_BK_AUTHORIZATION: json.dumps({"access_token": access_token.access_token}),
            }
        )
    except Exception:
        pass

    return RequestAPIClient(
        app_code=APP_CODE, app_secret=SECRET_KEY, headers=headers, common_args=common_args, stage=stage
    )



# -*- coding: utf-8 -*-


class BaseException(Exception):
    pass


class APIException(BaseException):
    """Exception for API"""

    def __init__(self, error_message, resp=None, url=""):
        self.url = url
        self.error_message = error_message
        self.resp = resp

        if self.resp is not None:
            error_message = "%s, resp=%s" % (error_message, self.resp.text)
        super(APIException, self).__init__(error_message)



# -*- coding: utf-8 -*-
from bkapi_client_core.apigateway import APIGatewayClient, Operation, OperationGroup, bind_property


class Group(OperationGroup):
    # bkapi resource add_related_apps
    # 添加网关关联应用
    add_related_apps = bind_property(
        Operation,
        name="add_related_apps",
        method="POST",
        path="/api/v1/apis/{api_name}/related-apps/",
    )

    # bkapi resource apply_permissions
    # 申请网关API访问权限
    apply_permissions = bind_property(
        Operation,
        name="apply_permissions",
        method="POST",
        path="/api/v1/apis/{api_name}/permissions/apply/",
    )

    # bkapi resource create_resource_version
    # 创建资源版本
    create_resource_version = bind_property(
        Operation,
        name="create_resource_version",
        method="POST",
        path="/api/v1/apis/{api_name}/resource_versions/",
    )

    # bkapi resource generate_sdk
    # 生成 SDK
    generate_sdk = bind_property(
        Operation,
        name="generate_sdk",
        method="POST",
        path="/api/v1/apis/{api_name}/sdk/",
    )

    # bkapi resource get_apigw_public_key
    # 获取网关公钥
    get_apigw_public_key = bind_property(
        Operation,
        name="get_apigw_public_key",
        method="GET",
        path="/api/v1/apis/{api_name}/public_key/",
    )

    # bkapi resource get_apis
    # 查询网关
    get_apis = bind_property(
        Operation,
        name="get_apis",
        method="GET",
        path="/api/v1/apis/",
    )

    # bkapi resource get_latest_resource_version
    # 获取网关最新版本
    get_latest_resource_version = bind_property(
        Operation,
        name="get_latest_resource_version",
        method="GET",
        path="/api/v1/apis/{api_name}/resource_versions/latest/",
    )

    # bkapi resource get_released_resources
    # 查询已发布资源列表
    get_released_resources = bind_property(
        Operation,
        name="get_released_resources",
        method="GET",
        path="/api/v1/apis/{api_name}/released/stages/{stage_name}/resources/",
    )

    # bkapi resource get_stages
    # 查询环境
    get_stages = bind_property(
        Operation,
        name="get_stages",
        method="GET",
        path="/api/v1/apis/{api_name}/stages/",
    )

    # bkapi resource get_stages_with_resource_version
    # 查询网关环境资源版本
    get_stages_with_resource_version = bind_property(
        Operation,
        name="get_stages_with_resource_version",
        method="GET",
        path="/api/v1/apis/{api_name}/stages/with-resource-version/",
    )

    # bkapi resource grant_permissions
    # 网关为应用主动授权
    grant_permissions = bind_property(
        Operation,
        name="grant_permissions",
        method="POST",
        path="/api/v1/apis/{api_name}/permissions/grant/",
    )

    # bkapi resource import_resource_docs_by_archive
    # 通过文档归档文件导入资源文档
    import_resource_docs_by_archive = bind_property(
        Operation,
        name="import_resource_docs_by_archive",
        method="POST",
        path="/api/v1/apis/{api_name}/resource-docs/import/by-archive/",
    )

    # bkapi resource import_resource_docs_by_swagger
    # 通过 Swagger 格式导入文档
    import_resource_docs_by_swagger = bind_property(
        Operation,
        name="import_resource_docs_by_swagger",
        method="POST",
        path="/api/v1/apis/{api_name}/resource-docs/import/by-swagger/",
    )

    # bkapi resource release
    # 发布版本
    release = bind_property(
        Operation,
        name="release",
        method="POST",
        path="/api/v1/apis/{api_name}/resource_versions/release/",
    )

    # bkapi resource sync_api
    # 同步网关
    sync_api = bind_property(
        Operation,
        name="sync_api",
        method="POST",
        path="/api/v1/apis/{api_name}/sync/",
    )

    # bkapi resource sync_resources
    # 同步资源
    sync_resources = bind_property(
        Operation,
        name="sync_resources",
        method="POST",
        path="/api/v1/apis/{api_name}/resources/sync/",
    )

    # bkapi resource sync_stage
    # 同步环境
    sync_stage = bind_property(
        Operation,
        name="sync_stage",
        method="POST",
        path="/api/v1/apis/{api_name}/stages/sync/",
    )

    # bkapi resource update_gateway_status
    # 修改网关状态
    update_gateway_status = bind_property(
        Operation,
        name="update_gateway_status",
        method="POST",
        path="/api/v1/apis/{api_name}/status/",
    )


class Client(APIGatewayClient):
    """Bkapi bk_apigateway client"""
    _api_name = "bk-apigateway"

    api = bind_property(Group, name="api")



# -*- coding: utf-8 -*-
from bkapi_client_core.apigateway import APIGatewayClient, Operation, OperationGroup


class Group(OperationGroup):

    @property
    def add_related_apps(self) -> Operation:
        """
        bkapi resource add_related_apps
        添加网关关联应用
        """

    @property
    def apply_permissions(self) -> Operation:
        """
        bkapi resource apply_permissions
        申请网关API访问权限
        """

    @property
    def create_resource_version(self) -> Operation:
        """
        bkapi resource create_resource_version
        创建资源版本
        """

    @property
    def generate_sdk(self) -> Operation:
        """
        bkapi resource generate_sdk
        生成 SDK
        """

    @property
    def get_apigw_public_key(self) -> Operation:
        """
        bkapi resource get_apigw_public_key
        获取网关公钥
        """

    @property
    def get_apis(self) -> Operation:
        """
        bkapi resource get_apis
        查询网关
        """

    @property
    def get_latest_resource_version(self) -> Operation:
        """
        bkapi resource get_latest_resource_version
        获取网关最新版本
        """

    @property
    def get_released_resources(self) -> Operation:
        """
        bkapi resource get_released_resources
        查询已发布资源列表
        """

    @property
    def get_stages(self) -> Operation:
        """
        bkapi resource get_stages
        查询环境
        """

    @property
    def get_stages_with_resource_version(self) -> Operation:
        """
        bkapi resource get_stages_with_resource_version
        查询网关环境资源版本
        """

    @property
    def grant_permissions(self) -> Operation:
        """
        bkapi resource grant_permissions
        网关为应用主动授权
        """

    @property
    def import_resource_docs_by_archive(self) -> Operation:
        """
        bkapi resource import_resource_docs_by_archive
        通过文档归档文件导入资源文档
        """

    @property
    def import_resource_docs_by_swagger(self) -> Operation:
        """
        bkapi resource import_resource_docs_by_swagger
        通过 Swagger 格式导入文档
        """

    @property
    def release(self) -> Operation:
        """
        bkapi resource release
        发布版本
        """

    @property
    def sync_api(self) -> Operation:
        """
        bkapi resource sync_api
        同步网关
        """

    @property
    def sync_resources(self) -> Operation:
        """
        bkapi resource sync_resources
        同步资源
        """

    @property
    def sync_stage(self) -> Operation:
        """
        bkapi resource sync_stage
        同步环境
        """

    @property
    def update_gateway_status(self) -> Operation:
        """
        bkapi resource update_gateway_status
        修改网关状态
        """


class Client(APIGatewayClient):
    """Bkapi bk_apigateway client"""

    @property
    def api(self) -> Group:
        """api resources"""



# -*- coding: utf-8 -*-
from .resources import CollectionsAPI

# 判断是否在Django环境下，如果在Django环境下，默认从settings中读取配置信息，否则，使用默认配置
# 客户端可采用import后再修改的方式来改变配置
try:
    from django.conf import settings

    APP_CODE = settings.APP_CODE
    SECRET_KEY = settings.SECRET_KEY
except Exception:
    APP_CODE = None
    SECRET_KEY = None

try:
    from bkapi_client_core.config import settings

    HOST = settings.get("BK_API_URL_TMPL", "")
except ImportError:
    HOST = ""

AVAILABLE_COLLECTIONS = {
    "api": CollectionsAPI,
}



# -*- coding: utf-8 -*-
from .base import RequestAPI


class CollectionsAPI(object):
    def __init__(self, client):
        from . import conf

        self.client = client
        self.host = conf.HOST.format(api_name="bk-apigateway")

        self.sync_api = RequestAPI(client=self.client, method="POST", host=self.host, path="/api/v1/apis/{api_name}/sync/")

        self.sync_stage = RequestAPI(client=self.client, method="POST", host=self.host, path="/api/v1/apis/{api_name}/stages/sync/")

        self.sync_resources = RequestAPI(client=self.client, method="POST", host=self.host, path="/api/v1/apis/{api_name}/resources/sync/")

        self.create_resource_version = RequestAPI(client=self.client, method="POST", host=self.host, path="/api/v1/apis/{api_name}/resource_versions/")

        self.apply_permissions = RequestAPI(client=self.client, method="POST", host=self.host, path="/api/v1/apis/{api_name}/permissions/apply/")

        self.get_apigw_public_key = RequestAPI(client=self.client, method="GET", host=self.host, path="/api/v1/apis/{api_name}/public_key/")

        self.get_latest_resource_version = RequestAPI(client=self.client, method="GET", host=self.host, path="/api/v1/apis/{api_name}/resource_versions/latest/")

        self.release = RequestAPI(client=self.client, method="POST", host=self.host, path="/api/v1/apis/{api_name}/resource_versions/release/")

        self.grant_permissions = RequestAPI(client=self.client, method="POST", host=self.host, path="/api/v1/apis/{api_name}/permissions/grant/")

        self.import_resource_docs_by_archive = RequestAPI(client=self.client, method="POST", host=self.host, path="/api/v1/apis/{api_name}/resource-docs/import/by-archive/")

        self.import_resource_docs_by_swagger = RequestAPI(client=self.client, method="POST", host=self.host, path="/api/v1/apis/{api_name}/resource-docs/import/by-swagger/")

        self.add_related_apps = RequestAPI(client=self.client, method="POST", host=self.host, path="/api/v1/apis/{api_name}/related-apps/")

        self.generate_sdk = RequestAPI(client=self.client, method="POST", host=self.host, path="/api/v1/apis/{api_name}/sdk/")

        self.get_apis = RequestAPI(client=self.client, method="GET", host=self.host, path="/api/v1/apis/")

        self.get_stages = RequestAPI(client=self.client, method="GET", host=self.host, path="/api/v1/apis/{api_name}/stages/")

        self.get_released_resources = RequestAPI(client=self.client, method="GET", host=self.host, path="/api/v1/apis/{api_name}/released/stages/{stage_name}/resources/")

        self.get_stages_with_resource_version = RequestAPI(client=self.client, method="GET", host=self.host, path="/api/v1/apis/{api_name}/stages/with-resource-version/")

        self.update_gateway_status = RequestAPI(client=self.client, method="POST", host=self.host, path="/api/v1/apis/{api_name}/status/")



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

readme = ''

setup(
    description=r'''蓝鲸API网关''',
    long_description=readme,
    name='bkapi-bk-apigateway',
    version='1.0.8',
    python_requires='!=3.0.*,!=3.1.*,!=3.2.*,!=3.3.*,!=3.4.*,!=3.5.*,<4.0,>=2.7',
    author='blueking',
    license='Apach License 2.0',
    packages=find_packages(),
    namespace_packages=find_packages(exclude=["*.*"]),
    package_dir={"": "."},
    package_data={"bkapi.bk_apigateway": ["py.typed", "*.pyi"]},
    install_requires=[
        'bkapi-client-core>=1.0.2,<2.0.0',
    ],
    extras_require={
        'django': ['bkapi-client-core[django]>=1.0.2,<2.0.0'],
    },
)



# -*- coding: utf-8 -*-
import logging

from .exceptions import APIException
from .utils import render_string

logger = logging.getLogger(__name__)


class RequestAPI(object):
    """Single request api"""

    HTTP_STATUS_OK = 200

    def __init__(self, client, method, host=None, path=None, description=""):
        self.host = host
        self.path = path
        self.client = client
        self.method = method

    def __call__(self, *args, **kwargs):
        try:
            return self._call(*args, **kwargs)
        except APIException as e:
            # Combine log message
            log_message = [
                e.error_message,
            ]
            log_message.append("url=%s" % e.url)
            if e.resp:
                log_message.append("content=%s" % e.resp.text)

            logger.exception("\n".join(log_message))

            # Try return error message from remote service
            if e.resp is not None:
                try:
                    return e.resp.json()
                except Exception:
                    pass
            return {"result": False, "message": e.error_message, "data": None}

    def _call(self, params=None, path_params=None):
        if self.method in ["GET", "HEAD"]:
            params, data = params, None
        elif self.method in ["POST", "PUT", "PATCH", "DELETE"]:
            params, data = None, params

        path = render_string(self.path, path_params) if path_params else self.path
        # Request remote server
        url = "%s/%s%s" % (self.host.rstrip("/"), self.client.stage, path)
        try:
            resp = self.client.request(self.method, url, params=params, data=data)
        except Exception as e:
            logger.exception("Error occurred when requesting method=%s, url=%s", self.method, url)
            raise APIException(u"API调用出错, Exception: %s" % str(e), url=url)

        # Parse result
        if resp.status_code != self.HTTP_STATUS_OK:
            message = u"请求出现错误，请求HTTP状态码：%s" % resp.status_code
            raise APIException(message, resp=resp, url=url)

        # Response format json or text
        try:
            return resp.json()
        except Exception:
            raise APIException(u"返回数据格式不正确，统一为json", resp=resp, url=url)



# -*- coding: utf-8 -*-
"""API Gateway Client
"""
import json
import logging

import requests

from .conf import APP_CODE, AVAILABLE_COLLECTIONS, SECRET_KEY

logger = logging.getLogger(__name__)


class BaseClient(object):
    """Base client class for api"""

    @classmethod
    def setup_resources(cls, resources):
        cls.available_collections = resources

    def __init__(self, app_code=None, app_secret=None, headers=None, common_args=None, stage="prod", timeout=None):
        """
        :param str app_code: App code to use
        :param str app_secret: App secret to use
        :param dict headers: headers be sent to api
        :param dict common_args: Args that will apply to every request
        :param str stage: Stage for api gateway
        :param int timeout: timeout for request
        """

        self.app_code = app_code or APP_CODE
        self.app_secret = app_secret or SECRET_KEY
        self.headers = headers or {}
        self.common_args = common_args or {}
        self.stage = stage
        self.timeout = timeout
        self._cached_collections = {}
        self._log_warning()

    def _log_warning(self):
        _, _, api_name = __package__.partition(".")
        logger.warning(
            "%s is no longer maintained, recommend to use bkapi.%s as a replacement",
            __package__,
            api_name,
        )

    def set_stage(self, stage):
        """Change the value of stage

        :param str stage: Stage for api gateway
        """
        self.stage = stage

    def set_timeout(self, timeout):
        self.timeout = timeout

    def merge_params_data_with_common_args(self, method, params, data):
        """Add common args to params every request"""
        common_args = dict(app_code=self.app_code, **self.common_args)
        if method in ["GET", "HEAD"]:
            _params = common_args.copy()
            _params.update(params or {})
            params = _params
        elif method in ["POST", "PUT", "PATCH", "DELETE"]:
            _data = common_args.copy()
            _data.update(data or {})
            data = json.dumps(_data)
        return params, data

    def __getattr__(self, key):
        if key not in self.available_collections:
            return getattr(super(BaseClient, self), key)

        if key not in self._cached_collections:
            collection = self.available_collections[key]
            self._cached_collections[key] = collection(self)
        return self._cached_collections[key]

    def request(self, method, url, params=None, data=None, path_params=None, **kwargs):
        pass


class RequestAPIClient(BaseClient):
    def request(self, *args, **kwargs):
        # update headers
        headers = kwargs.pop("headers", {})
        headers.update(self.headers)

        return self._request(headers=headers, *args, **kwargs)

    def _request(self, method, url, params=None, data=None, **kwargs):
        """Send request direct"""
        params, data = self.merge_params_data_with_common_args(method, params, data)
        logger.debug("Calling %s %s with params=%s, data=%s", method, url, params, data)
        return requests.request(method, url, params=params, data=data, timeout=self.timeout, **kwargs)


RequestAPIClient.setup_resources(AVAILABLE_COLLECTIONS)



# -*- coding: utf-8 -*-
from bkapi_client_core.apigateway import generic_type_partial as _partial
from bkapi_client_core.apigateway.django_helper import get_client_by_request as _get_client_by_request
from bkapi_client_core.apigateway.django_helper import get_client_by_username as _get_client_by_username

from .client import Client

get_client_by_request = _partial(Client, _get_client_by_request)
get_client_by_username = _partial(Client, _get_client_by_username)
